package state51::Job::AddFactToArtist;
use Moose;
use MooseX::Types::Moose qw/ Str Int /;
use Moose::Util::TypeConstraints;
use MooseX::ClassAttribute;
extends 'state51::Job';

class_has documentation => (
    is => 'ro',
    isa => 'Str',
    default => 'Add a fact, which can be a url or an identifier to an artist',
);

sub exchange_name {
    'add_fact_to_artist'
}

has 'artist_id' => (
    is      => 'ro',
    isa     => Int,
    documentation => 'The full instance id (including class id) of the artist to be operated on.',
    required => 1,
);

has 'fact' => (
    is      => 'ro',
    isa     => Str,
    required => 1,
    documentation => 'The fact to be added to the artist, e.g. twitter, wikipedia, facebook urls.',
);

sub suppress_console_display { 1 }

sub run {
    my ($self) = @_;
    state51::Base->connect;

    my ($artist) = state51::Artist->load_oids($self->artist_id);
    $artist->add_fact($self->fact);

    return 1;
}

with qw/
    state51::Mixin::JobLogger
/;

__PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;
