package state51::Job::CreateDeliverDesiresForWholeCatalogue;
use Moose;
use MooseX::Types::Moose qw/ArrayRef/;
use namespace::autoclean;

use aliased 'state51::Job::RequestTransformsForOrder';

extends 'state51::Job';

sub exchange_name { 'order_desires_for_whole_catalogue' }

sub run {
    my $self = shift;
    my $notes = '';
    my $correspondence = '';
    my $has_error = 0;
    my $cancelled = 0;
    eval {
        my ($product_count, $track_count, $invalid_dps) = $self->generate_desires_for_whole_catalogue("CreateDeliverDesiresForWholeCatalogue");
        $correspondence = $self->describe_delivery_point_validation_errors($invalid_dps) if (scalar keys %$invalid_dps);
        if($product_count) {
            my $no_deliverable_upcs = $self->validate_order_desires([$self->get_delivery_desires],$invalid_dps);
            $self->_order_object->log_message(
                level => 'info',
                message => [
                    RequestTransformsForOrder->new(
                        order => $self->order,
                    )
                ]
            );
            $notes .= $self->_order_object->describe_desire_validation_errors($no_deliverable_upcs);
            $self->_order_object->_set_status('ready');
            $self->_order_object->update;
            $notes .= $self->date . " Generated delivery desires for whole catalogue, $product_count products, $track_count tracks";
        }
        else {
            $cancelled = 1;
            $correspondence .= "\nNo products to be delivered in this order. This order will be cancelled";
            $self->_order_object->cancel_order('the CI System, due there is no products to be delivered','');
        }
    };
    if ($@) {
        $has_error = 1;
        $notes = $self->date . " Something wrong when procesing full catalogue delivery order: $@\n";
    }
    eval {
        #$self->_order_object->comment_in_rt($notes.$self->job_signature) if $notes;
        $self->_order_object->correspond_in_rt($correspondence.$self->job_signature) if $correspondence;
    };
    if ($@) {
        $has_error = 1;
        warn "CreateDeliverDesiresForWholeCatalogue: " . $notes;
        warn("CreateDeliverDesiresForWholeCatalogue: Could not comment into RT: $@");
    };

    if ($has_error) {
        $self->update_ticket_status('open');
    }
    elsif(!$cancelled) {
        $self->update_ticket_status('stalled');
    }
    print $notes if ($notes && $self->interactive);
    return !$has_error;
}

sub lockname { "whole_cat_dd_".$_[0]->_order_object->id };

with qw/
    state51::Script::Helper::InteractiveSuccess
    state51::Mixin::HandleOrdersOnJobs
    state51::Mixin::JobSignature
    state51::Mixin::LockedJob
    state51::Mixin::Loggable
/;

has '+_order_object' => (
    handles => {
        get_delivery_desires => 'get_delivery_desires',
        generate_desires_for_whole_catalogue => 'generate_deliver_desire_for_whole_catalogue',
        update_ticket_status => 'update_ticket_status',
        are_deliverypoints_for_deliver_desires_valid => 'are_deliverypoints_for_deliver_desires_valid',
        describe_delivery_point_validation_errors => 'describe_delivery_point_validation_errors',
        validate_order_desires => 'validate_order_desires',
    }
);

# FIXME - This breaks things, why?
#__PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;
