package state51::Job::DeleteTransferDetailKeys;
use Moose;
use MooseX::Types::Moose qw/ ArrayRef Str Int HashRef /;
extends 'state51::Job';


has 'transfer_detail_id' => (
    isa     => Int,
    is      => 'ro',
    default => sub { [] },
    documentation =>
'The instance id of the transfer details which relate to the public and private keys.'
);

sub exchange_name {
    'transfer_details_files'
}

sub suppress_console_display { 1 }

sub run {
    my ($self) = @_;
    state51::Base->connect;

    $self->remove_files_from_td();

    return 1;
}

sub remove_files_from_td {
    my ($self) = @_;

    my $td =
      state51::TransferDetails->load_polymorphic($self->transfer_detail_id());

    unless ($td->ssh_key_file_private and $td->ssh_key_file_public) {
        my $message = "cannot remove key files from a td that doesnt have both " . $td->id;
        $self->log_message(
            level   => 'fatal',
            message => [ $message ]
        );
        die($message);
    }
    $td->ssh_key_file_private->erase();
    $td->ssh_key_file_private(undef);

    $td->ssh_key_file_public->erase();
    $td->ssh_key_file_public(undef);
    $td->update();

    $self->log_message("Public and Private key files removed from " . $td->id());
}

__PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;
