package state51::Job::DeliverOneDPForOrder::DesireList;

use Moose;
use MooseX::Storage;
with Storage ('format' => 'JSON');
use Moose::Util::TypeConstraints;
use MooseX::Types::Moose qw/ Int ArrayRef /;
use Moose::Autobox;
use namespace::autoclean;

extends 'state51::Job::DeliverOneDPForOrder';

has 'deliver_desire_ids' => (
    is            => 'ro',
    isa           => ArrayRef[Int],
    required      => 1,
    documentation => 'an array of DeliverDesire IDs'
);

class_type 'state51::DeliverDesire';
has _deliver_desires => (
    is => 'ro',
    isa => ArrayRef['state51::DeliverDesire'],
    traits => [qw/ DoNotSerialize /],
    default => sub {[
        state51::DeliverDesire->load( shift->deliver_desire_ids->flatten )
    ]},
    lazy => 1,
);

sub get_formats_to_deliver {
    my $self = shift;

    map { $_->asset }
    grep { ! $_->asset->in_successful_delivery_to( musicservice => $_->entity, task => $_->request->task, fulfils_export_request => $_->request ) }
    grep { $_->status ne "undel" }
    $self->_deliver_desires->flatten;
}

sub can_trigger_more_delivery { 0 }

__PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;
