package state51::Job::EnsureFormatHasStandardEncode;

use MooseX::Types::Moose qw/Str/;
use state51::Moose;
use MooseX::Storage;
with Storage (format => "JSON");

use Try::Tiny;
use state51::Base;
use MooseX::Storage;

extends 'state51::Job';

with Storage('format' => 'JSON');

sub suppress_console_display { 1 }

has format_id => ( isa => Str, is => 'ro', required => 1 );

sub exchange_name { 'ensureformathasstandardencode' }

method lockname() { "ensureformathasstandardencode_" . $self->format_id }

method run() {
    state51::Base->connect();

    my $format = state51::ReleaseFormat->load($self->format_id);
    die "cannot load format with id <<".$self->format_id.">>" unless $format;
    die "format must have asset controller <<".$self->format_id.">>" unless $format->asset_controller;

    my @encs = (
        state51::Encoding->fetch_one( name => "aac_96c_stream_nometa" ),
        state51::Encoding->fetch_one( name => "essentia_mir_stats_json" ),
#        state51::Encoding->fetch_one( name => "fingerprint_chromaprint" ),
        state51::Encoding->fetch_one( name => "flac_8" ),
#        state51::Encoding->fetch_one( name => "mp3_128c" ),
#        state51::Encoding->fetch_one( name => "mp3_128c_30" ),
#        state51::Encoding->fetch_one( name => "mp3_320c" ),
#        state51::Encoding->fetch_one( name => "mp3_320c_30" ),
        state51::Encoding->fetch_one( name => "ripper wav files" ),
#        state51::Encoding->fetch_one( name => "vorbis_96c_stream_nometa" ),
        state51::Encoding->fetch_one( name => "wav_30" ),
#        state51::Encoding->fetch_one( name => "Speech Music Classifier" ),
    );
    my $request = $self->find_or_make_standard_request_for($format);
    foreach my $standard_encoding (@encs) {
        try {
            $format->assert_encoded_into($standard_encoding);
        }
        catch { # either it's untransformable or it's not got encodes
            try {
                $format->request_transforms_for_order($request, '', $standard_encoding);
            }
            catch {
                warn $_;
            };
        };
    }
    $self->ensure_format_has_standard_thumbnails($format);

    return 1;
}

method ensure_format_has_standard_thumbnails($format) {
    return unless $format->packshot();

    my $packshot = $format->packshot;
    my @thumbnails = state51::FileTransform->fetch(master => $packshot, target_transform => 'thumb-jpeg-400x400', status => 'complete');
    if (($packshot) and (not @thumbnails)) {
        eval {
            $packshot->resize_into_file(400, 400, undef, 'jpeg');
        };
    }
    @thumbnails = state51::FileTransform->fetch(master => $packshot, target_transform => 'thumb-jpeg-270x270', status => 'complete');
    if (($packshot) and (not @thumbnails)) {
        eval {
            $packshot->resize_into_file(270, 270, undef, 'jpeg');
        };
    }
    @thumbnails = state51::FileTransform->fetch(master => $packshot, target_transform => 'thumb-jpeg-55x55', status => 'complete');
    if (($packshot) and (not @thumbnails)) {
        eval {
            $packshot->resize_into_file(55, 55, undef, 'jpeg');
        };
    }
    @thumbnails = state51::FileTransform->fetch(master => $packshot, target_transform => 'thumb-jpeg-300x300', status => 'complete');
    if (($packshot) and (not @thumbnails)) {
        eval {
            $packshot->resize_into_file(300, 300, undef, 'jpeg');
        };
    }

    return $self;
}

method find_or_make_standard_request_for ($format) {
    my ($request) = state51::Request::Export->fetch(
        organisation => $format->asset_controller,

        external_id  => 'standard encodes'
    );
    if (not $request) {
        my $notes = "Internal order for the creation of standard encodes for format preview.";

        $request = state51::Request::Export->new(
            name          => 'standard encodes',
            external_id   => 'standard encodes',
            do_not_charge => 1,
            status        => 'complete',
            organisation  => $format->asset_controller,
            notes         => $notes,
            priority      => 'Catalogue',
        );
        $request->insert;
    }

    return $request;
}

with qw/
    state51::Mixin::Loggable
    state51::Mixin::LockedJob
    state51::Mixin::UnloadAfterJob
/;

__PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;

1;
