package state51::Job::FixBrokenFile::Action::File::ChangeEncoding;
use state51::Moose;
use Try::Tiny;
use state51::TransformStep;

extends qw/ state51::Job::FixBrokenFile::Action::File /;
use state51::TransformStep;

sub is_terminal { 0; }
has encoding => (
    isa => 'state51::Encoding',
    is => 'ro',
    required => 1,
);

sub full_message {
    my ($self) = @_;
    return $self->message."\n\nAction taken: Correcting the encoding of ".$self->file->id." and re-requesting the transforms.\n";
}

sub action {
    my ($self) = @_;

    my $we_originally_had = $self->file->encoding();

    $self->file->encoding($self->encoding);
    $self->file->update();

    foreach my $x ($self->file->master_of->flatten) {
        if ($x->status ne "junk") {
            $x->status('junk');
            $x->update();
        }
        my $replaced_by = state51::TransformStep::post_transforms_for_steps(
                [$x->encode->to],
                [$x->target_track->master_audio],
                $x->target_track,
                $x->target_transform,
        );
        foreach my $r (@{ $replaced_by }) {
            $x->to_fulfil->register_transform($r) if $x->to_fulfil();
            $r->upsert();
        }
    }

    try {
        my $replaced_by = state51::TransformStep::post_transforms_for_steps(
                [$we_originally_had],
                [$self->file],
                $self->file->tracks->[0] || $self->file->track,
                undef,
        );
        foreach my $r (@{ $replaced_by }) {
            # We're not registering the transform, as we only have the file at
            # this point, and no transform.  The next step may take ownership
            # of the transform, if it needs it.
            $r->upsert();
        }
    }
    catch {
        warn "Could not produce initial encoding: $_";
    };

    return;
}

__PACKAGE__->meta->make_immutable;
1;

