package state51::Job::FixBrokenFile::Action::File::Leave;
use state51::Moose;
use state51::File;

extends qw/ state51::Job::FixBrokenFile::Action::File /;

sub is_terminal { 1; }

sub full_message {
    my ($self) = @_;
    return
        $self->message
      . "\n\nMarking as unfixable transforms of file "
      . $self->file->id
      . " on completed orders\n";
}

sub action {
    my ($self) = @_;

    my $log = 0;
    if ($self->file->encoded_by_us) {
        my $transform_resulting_in_me =
          state51::FileTransform->fetch_one(result => $self->file);

        # check the master is ok
        # and if so get rid of me and re-request
        my @urls = $transform_resulting_in_me->master->mogile_paths();
        if ($transform_resulting_in_me->master->status eq 'live' and @urls) {
            $self->file->stored(FILE_STATUS_GONE);
            $transform_resulting_in_me->result(undef);
            $transform_resulting_in_me->status('pending');
            $transform_resulting_in_me->priority(250);
            $transform_resulting_in_me->update();
            $transform_resulting_in_me->_trigger_transform();
        }
    }
    else {
        foreach my $x (grep { $_->status ne "complete" }
            $self->file->master_of->flatten)
        {
            if (   $x->to_fulfil
                && $x->to_fulfil->status ne 'ready'
                && $x->to_fulfil->status ne 'unfixable')
            {
                $x->status('unfixable');
                $x->update();
                $log = 1;
            }
        }
    }

    return;
}

__PACKAGE__->meta->make_immutable;
1;

