package state51::Job::ImageResizeIntoFile;
use Moose;
use MooseX::Types::Moose qw/ Str Num/;
use Moose::Util::TypeConstraints;
use MooseX::ClassAttribute;
extends 'state51::Job';

class_has documentation => (
    is => 'ro',
    isa => 'Str',
    default => 'Generate a new `state51::File::Image` with a target size, shape and type from an existing image. If a thumbnail of the right type already exists, this job will do nothing.'
);

sub exchange_name {
    'image_resize_into_file'
}

has 'image_id' => (
    isa           => Num,
    is            => 'ro',
    documentation => 'The full instance id of the state51::File::Image to be resized.'
);

has 'width' => (
    isa           => Num,
    is            => 'ro',
    documentation => 'The width of the new image.'
);

has 'height' => (
    isa           => Num,
    is            => 'ro',
    documentation => 'The height of the new image.'
);

has 'modifier' => (
    isa           => Str,
    is            => 'ro',
    documentation => 'The modifier to be applied to the new image. This can set the ideal shape of the image. See the POD in `state51::File::Image` for the full set of options.'
);

has 'file_type' => (
    isa           => Str,
    is            => 'ro',
    documentation => 'The file type of the new image. e.g. `jpg`. See the POD in `state51::File::Image` for the full set of options.'
);

sub lockname() {
    my ($self) = @_;

    'resize'.$self->image_id.$self->width.$self->height.$self->modifier.$self->file_type;
}

sub suppress_console_display { 1 }

sub run {
    my ($self) = @_;
    state51::Base->connect;

    my $image = state51::File::Image->load($self->image_id);
    # if this explodes, the JobLogger Mixin will catch and log it
    $image->resize_into_file($self->wifth, $self->height, $self->modifier, $self->file_type);

    return 1;
}

with qw/
    state51::Mixin::JobLogger
    state51::Mixin::LockedJob
    state51::Mixin::UnloadAfterJob
/;

__PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;
