package state51::Job::InitiateFTPImport;

use state51::Moose;
use MooseX::Storage;
with Storage (format => "JSON");

use state51::Base;
use state51::ImportFeedHandler;

use MooseX::Types::Common::String qw/ NonEmptySimpleStr /;

extends 'state51::Job';

=head1 INITIATEFTPIMPORT

Run batch ingestion from MQ.

Takes

filePath        =>  '/import/ftphome/remote-control-records-p/import/57294224320192.done'
and

organisation_id =>  765848230016

=cut


has 'filePath' => (
    is            => 'ro',
    isa           => NonEmptySimpleStr,
    required      => 1,
);

has 'organisation_id' => (
    is            => 'ro',
    isa           => NonEmptySimpleStr,
    required      => 1,
);

sub exchange_name { 'initiateftpimport' }

method lockname() {
     return 'import_' . $self->organisation_id; 
}

method run() {
    state51::Base->connect;

    my $lockname = $self->lockname;
    $self->do_whilst_locked($lockname,
        sub {
            my $o_import_feed_handler = state51::ImportFeedHandler->new({
                import_request_finder_class => "state51::ImportBatchFactory::ImportRequestFinder::MakeOne",
                import_request_finder_config => {
                    organisation_id => $self->organisation_id,
                },
            });
            my $success = $o_import_feed_handler->process_one_batch({batch_directory => $self->filePath});            
        }
    )
    || do { if ($self->interactive) { warn("Already locked: $lockname") } };
    return 1;
}

with qw/
 
    state51::Script::Helper::InteractiveSuccess
    state51::Mixin::HasLocks
    state51::Mixin::JobLogger
    state51::Mixin::HasLogger
    state51::Mixin::UnloadAfterJob

/;

#
__PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;

1;
