package state51::Job::ManageArtist;
use Moose;
use MooseX::Types::Moose qw/ Str /;
use Moose::Util::TypeConstraints;
extends 'state51::Job';

has 'artist_name' => (
    isa           => Str,
    is            => 'ro',
    documentation => 'The name of the Artist.'
);

has 'action' => (
    is  => 'ro',
    isa => enum([qw/ create /]),
    documentation =>
"The action we are taking on the artist, currently only create is supported",
);

sub exchange_name {
    'manage_artist'
}

sub suppress_console_display { 1 }

sub run {
    my ($self) = @_;
    state51::Base->connect;

    $self->execute_action();

    return 1;
}

sub execute_action {
    my ($self) = @_;

    if ($self->action() eq 'create') {
        $self->log_message(
            "Job::ManageArtist called to create " . $self->artist_name());
        if ($self->artist_name()) {
            my $artist = state51::Artist->get_one($self->artist_name());
            if ($artist) {
                $self->log_message(
                    "Job::ManageArtist made or found " . $artist->id);
            }
        }
    }
}

with qw/
    state51::Mixin::JobLogger
/;

__PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;
