package state51::Job::ManageAsset;
use Moose;
use MooseX::Types::Moose qw/ Str Int Maybe Bool /;
use Moose::Util::TypeConstraints;
use MooseX::ClassAttribute;
extends 'state51::Job';

class_has documentation => (
    is => 'ro',
    isa => 'Str',
    default => 'Manage state51::Asset and subclasses - this is typically used to update a SoundRecording. Note that you must include all the attributes or the Job will not start.'
);

sub exchange_name {
    'manage_asset'
}

has 'action' => (
    is  => 'ro',
    isa => enum([qw/ update /]),
    required => 1,
    documentation =>
"The action we are taking on the asset, currently only `update` is supported",
);

has 'asset_id' => (
    is      => 'ro',
    isa     => Int,
    documentation => 'The full instance id (including class id) of the instance to be operated on.',
    required => 1,
);

has 'language_of_performance' => (
    is      => 'ro',
    isa     => Maybe[Int],
    required => 1,
    documentation => 'The full instance id (including class id) of the language of performance. Can be null.'
);

has 'has_vocals' => (
    is      => 'ro',
    isa     => Maybe[Bool],
    required => 1,
    documentation => 'Does the Recording contain any vocalisations. Can be null.'
);

has 'has_instruments' => (
    is      => 'ro',
    isa     => Maybe[Bool],
    required => 1,
    documentation => 'Does the recording contain an instrument of any kind? Can be null.'
);

sub suppress_console_display { 1 }

sub run {
    my ($self) = @_;
    state51::Base->connect;

    $self->execute_action();

    return 1;
}

sub execute_action {
    my ($self) = @_;

    if ($self->action() eq 'update') {
        $self->log_message(
            "Job::ManageAsset called to update " . $self->asset_id());
        if ($self->asset_id) {
            my ($asset) = state51::Asset->load_oids($self->asset_id);
            if ($asset) {
                my $lop;
                if ($self->language_of_performance) {
                    $lop = state51::Language->load($self->language_of_performance);
                }
                $asset->language_of_performance($lop);

                $asset->has_vocals($self->has_vocals);
                $asset->has_instruments($self->has_instruments);
                $asset->update();
            }
        }
    }
}

with qw/
    state51::Mixin::JobLogger
/;

__PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;
