package state51::Job::ManageAssetAddParty;
use Moose;
use MooseX::Types::Moose qw/ Str Int /;
use Moose::Util::TypeConstraints;
use MooseX::ClassAttribute;
extends 'state51::Job';

class_has documentation => (
    is      => 'ro',
    isa     => 'Str',
    default => 'Manage state51::Asset and call the method to add parties.'
);

sub exchange_name {
    'manage_asset_add_party';
}

has 'asset_id' => (
    is  => 'ro',
    isa => Int,
    documentation =>
'The full instance id (including class id) of the instance to be operated on.',
    required => 1,
);

has 'party_name' => (
    is            => 'ro',
    isa           => Str,
    documentation => 'The name or email address of the party to be added.',
    required      => 1,
);

has 'party_role_desc' => (
    is  => 'ro',
    isa => Str,
    documentation =>
'The identifier for the party role. e.g. `mfit_mastered_by`, or `composer`',
    required => 1,
);

has 'claimant' => (
    is  => 'ro',
    isa => Int,
    documentation =>
      'The full instance id of the organisation making the claim.',
    required => 1,
);

sub suppress_console_display { 1 }

sub run {
    my ($self) = @_;
    state51::Base->connect;

    $self->log_message(
        "Job::ManageAssetAddParty called on " . $self->asset_id());
    if ($self->asset_id) {
        my $asset = state51::Asset::Recording->load($self->asset_id);

        my $organisation = state51::Organisation->load_oids($self->claimant);

        $asset->add_party(
            party_name => $self->party_name,
            party_role_desc => $self->party_role_desc,
            claimant => $organisation,
            source => $organisation->media_catalogue(),
        );
    }

    return 1;
}

with qw/
    state51::Mixin::JobLogger
/;

__PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;
