package state51::Job::ManageLicensedExports;
use Moose;
use Moose::Util::TypeConstraints;
use MooseX::Types::Moose qw/ Str Int Bool /;
extends 'state51::Job';

sub exchange_name { 'managelex' }

has org_licence_content_from => (
    isa      => 'Int',
    is       => 'ro',
    required => 1,
);

has org_deliver_via => (
    isa      => 'Int',
    is       => 'ro',
    required => 1,
);

has action => (
    is  => 'ro',
    isa => enum([qw/ create /]),
    documentation =>
"The action we are taking on the licensed export config of the via org, currently only create is supported",
);

sub run {
    my ($self) = @_;
    state51::Base->connect;

    $self->execute_action();
}

sub execute_action {
    my ($self) = @_;

    if ($self->action() eq 'create') {
        $self->log_message(
            "Job::ManageLicensedExports called to create " .  $self->org_licence_content_from . " on " . $self->org_deliver_via
        );

        my $org_from = state51::Organisation->load_oids($self->org_licence_content_from);
        my $org_via  = state51::Organisation->load_oids($self->org_deliver_via);

        $org_via->add_license_content_from($org_from);
        $org_via->update();
    }
}

with qw/
    state51::Mixin::JobLogger
/;

__PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;
