package state51::Job::ManageMailService;
use MooseX::Types::Moose qw/ Int Str /;
use Moose;
use Moose::Util::TypeConstraints;
extends 'state51::Job';

has 'role_id' => (
    is            => 'ro',
    isa           => Int,
    documentation => "The state51::Role ID that we are acting on",
);

has 'mailservice_id' => (
    is            => 'ro',
    isa           => Int,
    documentation => "The state51::MailService ID that we are acting on",
);

has 'action' => (
    is  => 'ro',
    isa => enum( [qw/ subscribe unsubscribe resubscribe /] ),
    documentation =>
"The action we are taking on the state51::Role and state51::MailService, subscribe or unsubscribe",
);

sub exchange_name { 'managemailservice' }

sub run {
    my ($self) = @_;
    state51::Base->connect;

    my $role        = state51::Role->load( $self->role_id );
    my $mailservice = state51::MailService->load( $self->mailservice_id );

    eval {
        if ( $self->action eq 'subscribe' ) {
            $role->subscribe($mailservice);
        }
        elsif ( $self->action eq 'unsubscribe' ) {
            $role->unsubscribe($mailservice);
        }
        elsif ( $self->action eq 'resubscribe' ) {
            $role->resubscribe($mailservice);
        }
    };
    if ($@) {
        $self->log_message(
            message => [
                state51::Logger::Event::Generic->new(
                        message => "Failed to "
                      . $self->action
                      . " state51::Role("
                      . $self->role_id
                      . ") from state51::MailService("
                      . $self->mailservice_id . ")"
                )
            ],
            level => 'info'
        );
    }
    else {
        $self->log_message(
            message => [
                state51::Logger::Event::Generic->new(
                        message => "state51::Role("
                      . $self->role_id
                      . ") was "
                      . $self->action
                      . "d from state51::MailService("
                      . $self->mailservice_id . ")"
                )
            ],
            level => 'info'
        );
    }
}

with qw/
    state51::Mixin::JobLogger
/;

__PACKAGE__->meta->make_immutable;
1;
