package state51::Job::ManageMailout;
use MooseX::Types::Moose qw/ Num /;
use Moose;
use Moose::Util::TypeConstraints;
extends 'state51::Job';

has 'mailout_id' => (
    is            => 'ro',
    isa           => Num,
    documentation => "The state51::Mailout ID that we are acting on",
);

has 'action' => (
    is  => 'ro',
    isa => enum( [qw/ opened /] ),
    documentation =>
"The action we are taking on the state51::Mailout",
);

sub exchange_name { 'managemailout' }

sub run {
    my ($self) = @_;
    state51::Base->connect;

    my $mailout = state51::Mailout->load( $self->mailout_id );

    eval {
        if ( $self->action eq 'opened' ) {
            $mailout->inc_opened();
        }
    };
    if ($@) {
        $self->log_message(
            message => [
                state51::Logger::Event::Generic->new(
                        message => "Failed to set "
                      . $self->action
                      . " for state51::Mailout("
                      . $self->mailout_id . ")"
                )
            ],
            level => 'info'
        );
    }
    else {
        $self->log_message(
            message => [
                state51::Logger::Event::Generic->new(
                        message => "Successfully set "
                      . $self->action
                      . " for state51::Mailout("
                      . $self->mailout_id . ")"
                )
            ],
            level => 'info'
        );
    }
}

with qw/
    state51::Mixin::JobLogger
/;

__PACKAGE__->meta->make_immutable;
1;
