package state51::Job::ManageStatusForCompliance;
use Moose;
use MooseX::Types::Moose qw/ Int Str /;
use Moose::Util::TypeConstraints qw/ enum /;
use MooseX::ClassAttribute;
use state51::Logger;
use state51::Logger::Event::SignupStateChange;
extends 'state51::Job';

class_has documentation => (
    is  => 'ro',
    isa => 'Str',
    default =>
'This job is used to manage the status of an organisation and roles undergoing compliance.',
);

sub exchange_name {
    'manage_status_for_compliance';
}

has 'organisation_id' => (
    is            => 'ro',
    isa           => Int,
    required      => 1,
    documentation => 'The id of the organisation to be managed.',
);

has 'target_status' => (
    is            => 'ro',
    isa           => enum([qw/ live junk /]),
    required      => 1,
    documentation => 'The status to which the role and organisation will be set.',
);

has 'modify_time' => (
    is            => 'ro',
    isa           => Str,
    required      => 1,
    documentation => 'The time at which the job was raised, if this is before the modify_time on the org, the job will be discarded.',
);

sub suppress_console_display { 1 }

sub run {
    my ($self) = @_;
    state51::Base->connect;

    my $organisation = state51::Base->load_oids($self->organisation_id);
    if (($organisation->source->id eq '85788250840009') or ($organisation->source->id eq '85788250850009')) {
        # FIXME: validate that the modify_time on the org is < $self->modify_time()
        $organisation->status($self->target_status);
        $organisation->update;

        my @roles = state51::Role->fetch(
            organisation_reps => [$organisation],
            includes => 'organisation_reps',
        );
        my $role_id;
        foreach my $role (@roles) {
            # things are going to get strange if roles rep multiple orgs
            $role->status($self->target_status);
            $role->update;
            $role_id = $role->id;
        }

        state51::Logger->new->message(
            level   => 'trace',
            message => [
                state51::Logger::Event::SignupStateChange->new(
                    current_value   => $self->target_status,
                    organisation_id => $organisation->id,
                    role_id         => $role_id,
                )
            ]
        );
    }

    return 1;
}

with qw/
  state51::Mixin::JobLogger
  /;

__PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;
