package state51::Job::OrderValidateAndCreateDeliverDesires;
use state51::Base;
use state51::Moose;
use MooseX::Storage;
with Storage (format => "JSON");

use Try::Tiny;
use MooseX::Types::Moose qw/ Int Bool Undef/;
use MooseX::Types::Common::String qw/ NonEmptySimpleStr /;
use namespace::autoclean;

extends 'state51::Job';

has file_no => (
    is => 'ro',
    isa => Int|Undef,
    documentation => 'The uploaded XLS file number to be validated',
    default => undef,
);

has metadata_file_no => (
    is => 'ro',
    isa => Int|Undef,
    documentation => 'The uploaded Service Metadata XLS file number to be validated',
    default => undef,
);

has ppl_file_no => (
    is => 'ro',
    isa => Int|Undef,
    documentation => 'The uploaded PPL Metadata XLS file number to be validated',
    default => undef,
);

has ignore_validation_errors => (
    is => 'ro',
    isa => Bool,
    default => 0,
    documentation => 'Ignore spreadhseet validation errors, just generate the known good products',
);

method run() {
    $0 = 'perl jobserver_worker: OrderValidateAndCreateDeliverDesires for ' . $self->order;
    #This to avoid breakage for already posted orders waiting for this old task to be run
    return state51::Job::CreateDeliverDesiresForXls->new(
        file_no => $self->file_no,
        ignore_validation_errors => $self->ignore_validation_errors,
        async => 0,
    )->run();
}

method lockname() { "order_validate_".$self->order }

with qw/
    state51::Mixin::Loggable
    state51::Mixin::HandleOrdersOnJobs
    state51::Mixin::RunsAsyncJobs
    state51::Mixin::JobSignature
    state51::Script::Helper::InteractiveSuccess
    state51::Mixin::LockedJob
/;

#
# __PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;
