package state51::Job::PostTransferStatusUpdate;
use state51::Base;
use state51::Moose;
use MooseX::Storage;
with Storage (format => "JSON");

use Try::Tiny;
use Moose::Util::TypeConstraints;
use MooseX::Types::Common::String qw/ NonEmptySimpleStr /;
use Moose::Autobox;
use Capture::Tiny qw/ capture_merged /;
use MooseX::Types::DateTime::MySQL ();
use MooseX::Types::DateTime qw/ DateTime /;
use DateTime::Duration;
use aliased DateTime => 'DT';
use 5.010;
use namespace::autoclean;

extends 'state51::Job';

has 'batch_id' => (
    is            => 'ro',
    isa           => NonEmptySimpleStr,
    required      => 1,
    documentation => 'A batch id',
);

class_type 'state51::ExportBatch';
has _batch => (
    is => 'ro',
    isa => 'state51::ExportBatch',
    traits => [qw/ DoNotSerialize /],
    default => sub {
        my $self = shift;
        state51::Base->connect;
        state51::ExportBatch->load($self->batch_id);
    },
    clearer => '_clear_batch',
    lazy => 1,
);


sub exchange_name {
    return "posttransferstatusupdate";
}

method lockname() {
    $self->_batch->musicservice->serialize_deliveries
        ? "order_transfer_".$self->_batch->musicservice->id
        : "order_transfer_".$self->_batch->id;
}

method run() {
    state51::Base->connect;
    my $lockname = $self->lockname;
    $self->do_whilst_locked($lockname,
        sub {
            my $order = $self->_batch->to_fulfil;
            my $my_hostname = (split('\.',Sys::Hostname::hostname()))[0];

            # FIXME - Duplicate code with DeliverOneDPForOrder
            my ($calculated_status, $all_deliveries_status, $delivery_status) = $order->calculate_status;
            my $full_comments = sprintf("%s Batch status update (%s) for order %s to %s executed at %s has finished\n\nOrder delivery status: %s\n",
                                 $self->date, $self->batch_id, $order->id, $self->_batch->musicservice->tla, $my_hostname, $all_deliveries_status);
            foreach my $name (sort keys %$delivery_status) {
                $full_comments .= "       $name: " . $delivery_status->{$name} . "\n";
            }

            if ('complete' eq $calculated_status) {
                $order->complete_order('automatic delivery script ' . $self->freeze, $my_hostname);
            }
            elsif ($self->_batch->upload_attempts < 3) {
                $self->run_job('state51::Job::RetryTransferForBatch', batch_id => $self->batch_id);
            }
            $self->_clear_batch; # FIXME - WHY!?!
            $self->_batch->notes($full_comments);
            $self->_batch->upsert;
            return;
        }
    )
    || do { if ($self->interactive) { warn("Already locked: $lockname") } };
    return 1;
}

with qw/
    state51::Script::Helper::InteractiveSuccess
    state51::Mixin::Loggable
    state51::Mixin::JobSignature
    state51::Mixin::RunsAsyncJobs
    state51::Mixin::HasLocks
    state51::Mixin::JobLogger
/;

#
__PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;
