package state51::Job::SetUpYouTubeTransferDetails;
use Moose;
use Moose::Util::TypeConstraints;
use MooseX::Types::Moose qw/ Str Int Bool /;
extends 'state51::Job';

sub exchange_name { 'setupyoutubetransferdetails' }

has org_id => (
    isa => 'Int',
    is => 'ro',
    required => 1,
);

has dp_code => (
    isa => 'Str',
    is => 'ro',
    required => 1,
);

has [qw/ audioswap override_manual_edits /] => (
    isa => 'Bool',
    is => 'ro',
    default => 0,
);

has [qw/ available_action not_available_action /] => (
    isa => enum([qw/ track monetize block /]),
    is => 'ro',
    required => 1,
);

has ownership_level => (
    isa => enum([qw/ recording recording_administers_publishing /]),
    is => 'ro',
    required => 1,
);

sub run {
    my ($self) = @_;
    state51::Base->connect;

    my $org = state51::Organisation->load_oids($self->org_id);
    my $dms = state51::OrganisationAttribute::MusicService->fetch_one( tla => $self->dp_code );

    my $available_action = $self->available_action;
    my $not_available_action = $self->not_available_action;
    my $audioswap = $self->audioswap;
    my $override = $self->override_manual_edits;
    my $ownership_level = $self->ownership_level;
    my $right_type = $self->ownership_level;
    # Note, no = recording
    #       yes = recording_administers_publishing

    $self->log_message(
        level => "info",
        message => [
            state51::Logger::Event::Generic->new(
                message => "available action: $available_action"
                . "not available action: $not_available_action"
                . "audioswap: ". ($audioswap ? "yes" : "no")
                . "override manual edits: ". ($override ? "yes" : "no")
                . "ownership level: $ownership_level"
            )
        ]
    );

    my $td = state51::TransferDetails->fetch_one( destination => $dms, organisation => $org );

    my @rules = state51::DMSPriceTierMap->fetch(rights_holder => $org,
                                        musicservice  => $dms,
                                    );
    foreach my $to_go (@rules) {
        next if $to_go->barcode;
        $to_go->erase;
    }

    my $map = state51::DMSPriceTierPolicy->new(
        musicservice => $dms,
        rights_holder => $org,
    )->make_new_map(
        result       => $available_action,
        track_result => $available_action,
    );

    $td->extra_config(state51::TransferDetails::ExtraConfig::YouTubeAudio->new(
        audioswap => $audioswap,
        no_stream_action => $not_available_action,
        ownership_level => $ownership_level,
        override_manual_edits => $override,
    ));
    $td->update;
}

__PACKAGE__->meta->make_immutable;
1;
