package state51::Job::WithdrawAllFormatsForAC;
use Moose;
use state51::Types qw/InstanceID/;
use Moose::Util::TypeConstraints;
use MooseX::ClassAttribute;
use List::Util qw(shuffle);

extends 'state51::Job';

class_has documentation => (
    is  => 'ro',
    isa => 'Str',
    default =>
'Mark all ReleaseFormats for a given asset controller as withdrawn.',
);

sub exchange_name {
    'withdraw_all_formats_for_ac';
}

has 'asset_controller_id' => (
    is       => 'ro',
    isa      => InstanceID,
    required => 1,
    documentation =>
      'The full instance id of the asset controller which will have formats withdrawn.',
);

# sub suppress_console_display { 1 }

sub run {
    my ($self) = @_;
    state51::Base->connect;

    my $asset_controller = state51::Organisation->load_oids($self->asset_controller_id);

    # This will not find Format::Supplementary
    # But each format is responsible for making sure it's supplementary formats are withdrawn
    my $format_remote = state51::ReleaseFormat->remote;
    my $filter = (
        $format_remote->{status}->one_of([qw/ live withdrawn pending /]) &
        $format_remote->{asset_controller} eq $asset_controller
    );

    my @formats = state51::Base->select_readonly(
        $format_remote,
        filter => $filter,
    );

    foreach my $format (shuffle @formats) {
        # state51::Logger::Event::AttributeChanged will be emitted per format once withdrawn
        $format->make_withdrawn('WithdrawAllFormatsForAC'); # supposed to pass username of withdrawer
        $format->unload();
    }

    return 1;
}

with qw/
  state51::Mixin::JobLogger
  /;

__PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;
