package state51::Mixin::HasLocks;
use Moose::Role;
use Carp qw/ confess cluck /;
use namespace::autoclean;
use state51::Logger::Event::LockClaimed;
use state51::Logger::Event::LockReleased;
use state51::Logger::Event::LockFailed;
use Time::HiRes qw/ gettimeofday tv_interval /;
use state51::Logger;

# There is a default timeout of 0 seconds here
# We get the lock or immediatly fail.

sub do_whilst_locked {
    my ($self, $lock_name, $sub, @args) = @_;

    confess "need a lock_name!" unless $lock_name;
    confess "sub is not a code ref!" unless (ref($sub) eq 'CODE');

    # MySQL enforces a maximum length on lock names of 64 characters.
    if (length($lock_name) > 59) { # prepended with lock_
        cluck("lock name $lock_name exceeds the allowed length. truncating");
        $lock_name = substr($lock_name, 0, 59);
    }

    my $logger = state51::Logger->new();

    my $dbh = state51::Base->tangram_storage->dbh_readwrite;
    my ($got) = $dbh->selectrow_array("SELECT GET_LOCK(?,0)", {}, "lock_$lock_name");

    if ($got==1) {
        my $t0 = [gettimeofday];
        $logger->message(level => "trace", message => [state51::Logger::Event::LockClaimed->new( name => $lock_name )]);

        my $fail;

        my $ret = eval {
            return $sub->(@args);
        };
        if ($@) {
            $fail = $@;
            $dbh->do("SELECT RELEASE_LOCK(?)", {}, "lock_$lock_name");
            die $fail;
        }

        $dbh->do("SELECT RELEASE_LOCK(?)", {}, "lock_$lock_name");
        $logger->message(level => "trace", message => [state51::Logger::Event::LockReleased->new( name => $lock_name, duration => tv_interval($t0, [gettimeofday]) )]);

        return $ret unless $fail;
    }
    else {
        $logger->message(level => "trace", message => [state51::Logger::Event::LockFailed->new( name => $lock_name )]);
    }

    return;
}

1;

