use strict;
use warnings;

use Test::More;
use Path::Class;
use File::Find;
use Class::Load;

use FindBin;
use File::Spec;
use lib File::Spec->catdir($FindBin::Bin, "lib");

BEGIN { $ENV{PLATFORM_MQ_SERVER} ||= 'localhost' }

# http://www.simplicidade.org/notes/archives/2009/11/perl_testing_an.html
# Test::Compile might also be useful

my $lib = dir('lib')->absolute;

if ($lib->can('resolve')) {
    $lib = $lib->resolve;
}

my $failed = 0;
find({
  bydepth => 1,
  no_chdir => 1,
  wanted => sub {
    my $m = $_;
    return unless $m =~ s/[.]pm$//;

    my ($volume, $directories, $file) = File::Spec->splitpath($m);
    my @dirs = File::Spec->splitdir($directories);
    pop @dirs; # last entry's blank

    while (@dirs && shift(@dirs) ne 'lib') { };

    $m = join("::", @dirs, $file);

    local $@;
    ok( eval { Class::Load::load_class($m); 1 }, "$m compiled " . $@||'') or do { $failed = 1 };
  },
}, $lib);

BAIL_OUT("COMPILE FAILED") if $failed;

done_testing();
